﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Operators;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Algorithms.ScatterSearch {
  /// <summary>
  /// A base class for operators that perform diversity calculation between two items.
  /// </summary>
  [Item("DiversityCalculator", "A base class for operators that perform diversity calculation between two items.")]
  [StorableClass]
  public abstract class DiversityCalculator : SingleSuccessorOperator {
    [StorableConstructor]
    protected DiversityCalculator(bool deserializing) : base(deserializing) { }
    protected DiversityCalculator(DiversityCalculator original, Cloner cloner) : base(original, cloner) { }
    protected DiversityCalculator() : base() { }

    public sealed override IOperation Apply() { // TODO: check if sealed fits here?
      return base.Apply();
    }

    public int ExecuteCalculation(IItem left, IItem right) {
      if (left == null || right == null) throw new ArgumentException("Cannot calculate diversity because one of the provided objects is null.");
      if (left.GetType() != right.GetType()) throw new ArgumentException("Cannot calculate diversity because the provided objects differ in type.");
      if (left == right) return 0;
      else return CalculateDiversity(left, right);
    }

    protected abstract int CalculateDiversity(IItem left, IItem right);
  }
}
