﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.BinaryVectorEncoding;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Algorithms.ScatterSearch.Knapsack {
  /// <summary>
  /// A base class for operators that perform a crossover of bool-valued vectors.
  /// </summary>
  [Item("NBinaryVectorCrossover", "A base class for operators that perform a crossover of bool-valued vectors.")]
  [StorableClass]
  public abstract class NBinaryVectorCrossover : SingleSuccessorOperator, INBinaryVectorCrossover, IStochasticOperator {
    public override bool CanChangeName {
      get { return false; }
    }

    #region Parameter properties
    public ScopeParameter CurrentScopeParameter {
      get { return (ScopeParameter)Parameters["CurrentScope"]; }
    }
    public ILookupParameter<IRandom> RandomParameter {
      get { return (ILookupParameter<IRandom>)Parameters["Random"]; }
    }
    public ILookupParameter<ItemArray<BinaryVector>> ParentsParameter {
      get { return (IScopeTreeLookupParameter<BinaryVector>)Parameters["Parents"]; }
    }
    #endregion

    #region Properties
    private IScope CurrentScope {
      get { return CurrentScopeParameter.ActualValue; }
    }
    #endregion

    [StorableConstructor]
    protected NBinaryVectorCrossover(bool deserializing) : base(deserializing) { }
    protected NBinaryVectorCrossover(NBinaryVectorCrossover original, Cloner cloner) : base(original, cloner) { }
    protected NBinaryVectorCrossover()
      : base() {
      #region Create parameters
      Parameters.Add(new ScopeParameter("CurrentScope"));
      Parameters.Add(new LookupParameter<IRandom>("Random"));
      Parameters.Add(new ScopeTreeLookupParameter<BinaryVector>("Parents"));
      #endregion
      ParentsParameter.ActualName = "BinaryVector";
    }

    public sealed override IOperation Apply() {
      var offspringSolutions = Cross(RandomParameter.ActualValue, ParentsParameter.ActualValue);
      for (int i = 0; i < offspringSolutions.Length; i++) {
        CurrentScope.Variables.Add(new Variable(string.Format("Child {0}", i), offspringSolutions[i]));
      }
      return base.Apply();
    }

    protected abstract ItemArray<BinaryVector> Cross(IRandom random, ItemArray<BinaryVector> parents);
  }
}
